@@ -6,10 +6,11 @@
 
 package Rex::Cloud::OpenStack;
 
+use 5.010001;
 use strict;
 use warnings;
 
-our $VERSION = '1.12.0'; # VERSION
+our $VERSION = '1.13.2'; # VERSION
 
 use Rex::Logger;
 
@@ -54,7 +55,7 @@ sub _request {
   Rex::Logger::debug("  $_ => $params{$_}") for keys %params;
 
   {
-    no strict 'refs'; ## no critic ProhibitNoStrict
+    no strict 'refs';
     $response = $self->{_agent}->request( $method->( $url, %params ) );
   }
 
