@@ -108,10 +108,11 @@ This module is the core commands module.
 
 package Rex::Commands;
 
+use 5.010001;
 use strict;
 use warnings;
 
-our $VERSION = '1.12.0'; # VERSION
+our $VERSION = '1.13.2'; # VERSION
 
 require Rex::Exporter;
 use Rex::TaskList;
@@ -1079,7 +1080,9 @@ sub needs {
 }
 
 # register needs in main namespace
-{
+_register_needs_in_main_namespace();
+
+sub _register_needs_in_main_namespace {
   my ($caller_pkg) = caller(1);
 
   if ( !$caller_pkg ) {
@@ -1090,7 +1093,7 @@ sub needs {
     my $ref_to_needs = qualify_to_ref( 'needs', 'main' );
     *{$ref_to_needs} = \&needs;
   }
-};
+}
 
 =head2 include Module::Name
 
@@ -1201,7 +1204,7 @@ With the LOCAL function you can do local commands within a task that is defined
 
 =cut
 
-sub LOCAL (&) { ## no critic ProhibitSubroutinePrototypes
+sub LOCAL (&) {
   my $cur_conn      = Rex::get_current_connection();
   my $local_connect = Rex::Interface::Connection->create("Local");
 
@@ -1952,7 +1955,7 @@ sub FALSE {
   return 0;
 }
 
-sub make(&) { ## no critic ProhibitSubroutinePrototypes
+sub make(&) {
   return $_[0];
 }
 
