@@ -49,7 +49,8 @@ With this module you can manipulate files.
    owner  => "root",
    group  => "root",
    mode  => 400,
-   on_change => sub { say "File was changed."; };
+   on_change => sub { say shift, " was changed."; },
+   on_no_change => sub { say shift, " wasn't changed."; };
 
 
 =head1 EXPORTED FUNCTIONS
@@ -58,11 +59,12 @@ With this module you can manipulate files.
 
 package Rex::Commands::File;
 
+use 5.010001;
 use strict;
 use warnings;
 use Fcntl;
 
-our $VERSION = '1.9.0'; # VERSION
+our $VERSION = '1.13.2'; # VERSION
 
 require Rex::Exporter;
 use Data::Dumper;
@@ -71,6 +73,7 @@ use Rex::FS::File;
 use Rex::Commands::Upload;
 use Rex::Commands::MD5;
 use Rex::File::Parser::Data;
+use Rex::Helper::File::Spec;
 use Rex::Helper::System;
 use Rex::Helper::Path;
 use Rex::Hook;
@@ -98,6 +101,10 @@ use vars qw(%file_handles);
 
 Parse a template and return the content.
 
+By default, it uses L<Rex::Template>. If any of the L<template_ng|Rex#template_ng> or L<1.3|Rex#1.3> (or newer) feature flag is enabled, then L<Rex::Template::NG> is used instead of this module (recommended).
+
+For more advanced functionality, you may use your favorite template engine via the L<set_template_function|Rex::Config#set_template_function> configuration option.
+
 =head3 Embedded templates
 
 Use C<__DATA__> to embed templates at the end of the file. Prefix embedded template names with C<@>. If embedding multiple templates, mark their end with C<@end>.
@@ -296,7 +303,8 @@ This function is the successor of I<install file>. Please use this function to u
      owner  => "user",
      group  => "group",
      mode   => 700,
-     on_change => sub { say "Something was changed." };
+     on_change => sub { say "Something was changed." },
+     on_no_change => sub { say "Nothing has changed." };
  
    file "/etc/motd",
      content => `fortune`;
@@ -363,25 +371,33 @@ you use Perl packages.
 All the possible variables ('{environment}', '{hostname}', ...) are documented
 in the CMDB YAML documentation.
 
-This function supports the following hooks:
+=head3 Hooks
 
-=over 8
+This function supports the following L<hooks|Rex::Hook>:
+
+=over 4
 
 =item before
 
-This gets executed before everything is done. The return value of this hook overwrite the original parameters of the function-call.
+This gets executed before anything is done. All original parameters are passed to it, including the applied defaults (C<ensure =E<gt> 'present'>, resolved path for C<source>).
+
+The return value of this hook overwrites the original parameters of the function call.
 
 =item before_change
 
-This gets executed right before the new file is written. Only with I<content> parameter. For the I<source> parameter the hook of the upload function is used.
+This gets executed right before the new file is written. All original parameters are passed to it, including the applied defaults (C<ensure =E<gt> 'present'>, resolved path for C<source>).
+
+Only called when the C<content> parameter is used. For the C<source> parameter, the L<upload|Rex::Commands::Upload#upload> hooks are used.
 
 =item after_change
 
-This gets executed right after the file was written. Only with I<content> parameter. For the I<source> parameter the hook of the upload function is used.
+This gets executed right after the file is written. All original parameters, including the applied defaults (C<ensure =E<gt> 'present'>, resolved path for C<source>), and any returned results are passed to it.
+
+Only called when the C<content> parameter is used. For the C<source> parameter, the L<upload|Rex::Commands::Upload#upload> hooks are used.
 
 =item after
 
-This gets executed right before the file() function returns.
+This gets executed right before the C<file()> function returns. All original parameters, including the applied defaults (C<ensure =E<gt> 'present'>, resolved path for C<source>), and any returned results are passed to it.
 
 =back
 
@@ -428,7 +444,7 @@ sub file {
 
   #### check and run before hook
   eval {
-    my @new_args = Rex::Hook::run_hook( file => "before", @_ );
+    my @new_args = Rex::Hook::run_hook( file => "before", $file, %{$option} );
     if (@new_args) {
       ( $file, @options ) = @new_args;
       $option = {@options};
@@ -442,15 +458,18 @@ sub file {
   Rex::get_current_connection()->{reporter}
     ->report_resource_start( type => "file", name => $file );
 
-  my $need_md5  = ( $option->{"on_change"} && !$is_directory ? 1 : 0 );
-  my $on_change = $option->{"on_change"} || sub { };
+  my $need_md5     = ( $option->{"on_change"} && !$is_directory ? 1 : 0 );
+  my $on_change    = $option->{"on_change"}    || sub { };
+  my $on_no_change = $option->{"on_no_change"} || sub { };
 
   my $__ret = { changed => 0 };
 
   my ( $new_md5, $old_md5 );
-  eval { $old_md5 = md5($file); };
 
-  if ( exists $option->{no_overwrite} && $option->{no_overwrite} && $old_md5 ) {
+  if ( exists $option->{no_overwrite}
+    && $option->{no_overwrite}
+    && $fs->is_file($file) )
+  {
     Rex::Logger::debug(
       "File already exists and no_overwrite option given. Doing nothing.");
     $__ret = { changed => 0 };
@@ -467,13 +486,7 @@ sub file {
     # first upload file to tmp location, to get md5 sum.
     # than we can decide if we need to replace the current (old) file.
 
-    my @splitted_file = split( /[\/\\]/, $file );
-    my $file_name     = ".rex.tmp." . pop(@splitted_file);
-    my $tmp_file_name = (
-      $#splitted_file != -1
-      ? ( join( "/", @splitted_file ) . "/" . $file_name )
-      : $file_name
-    );
+    my $tmp_file_name = get_tmp_file_name($file);
 
     my $fh    = file_write($tmp_file_name);
     my @lines = split( qr{$/}, $option->{"content"} );
@@ -483,6 +496,7 @@ sub file {
     $fh->close;
 
     # now get md5 sums
+    eval { $old_md5 = md5($file); };
     $new_md5 = md5($tmp_file_name);
 
     if ( $new_md5 && $old_md5 && $new_md5 eq $old_md5 ) {
@@ -507,7 +521,7 @@ sub file {
       );
 
       #### check and run before_change hook
-      Rex::Hook::run_hook( file => "before_change", @_ );
+      Rex::Hook::run_hook( file => "before_change", $file, %{$option} );
       ##############################
 
       if (Rex::is_sudo) {
@@ -532,7 +546,7 @@ sub file {
       );
 
       #### check and run after_change hook
-      Rex::Hook::run_hook( file => "after_change", @_, $__ret );
+      Rex::Hook::run_hook( file => "after_change", $file, %{$option}, $__ret );
       ##############################
 
     }
@@ -544,6 +558,8 @@ sub file {
 
     # HOOKS: for this case you have to use the upload hooks!
     $__ret = upload $option->{"source"}, "$file", force => 1;
+
+    $need_md5 = 0 if $__ret->{changed} == 0;
   }
 
   if ( exists $option->{"ensure"} ) {
@@ -551,7 +567,7 @@ sub file {
       if ( !$fs->is_file($file) ) {
 
         #### check and run before_change hook
-        Rex::Hook::run_hook( file => "before_change", @_ );
+        Rex::Hook::run_hook( file => "before_change", $file, %{$option} );
         ##############################
 
         my $fh = file_write($file);
@@ -565,7 +581,10 @@ sub file {
         );
 
         #### check and run after_change hook
-        Rex::Hook::run_hook( file => "after_change", @_, $__ret );
+        Rex::Hook::run_hook(
+          file => "after_change",
+          $file, %{$option}, $__ret
+        );
         ##############################
 
       }
@@ -576,12 +595,9 @@ sub file {
     }
     elsif ( $option->{ensure} eq "absent" ) {
       $need_md5 = 0;
-      delete $option->{mode};
-      delete $option->{group};
-      delete $option->{owner};
 
       #### check and run before_change hook
-      Rex::Hook::run_hook( file => "before_change", @_ );
+      Rex::Hook::run_hook( file => "before_change", $file, %{$option} );
       ##############################
 
       if ( $fs->is_file($file) ) {
@@ -606,7 +622,7 @@ sub file {
       }
 
       #### check and run after_change hook
-      Rex::Hook::run_hook( file => "after_change", @_, $__ret );
+      Rex::Hook::run_hook( file => "after_change", $file, %{$option}, $__ret );
       ##############################
 
     }
@@ -623,7 +639,7 @@ sub file {
         $dir_option{mode} = $option->{mode};
       }
 
-      Rex::Commands::Fs::mkdir( $file, %dir_option );
+      Rex::Commands::Fs::mkdir( $file, %dir_option, on_change => $on_change );
     }
   }
 
@@ -636,7 +652,7 @@ sub file {
     if ( !$fs->is_file($file) && !$is_directory ) {
 
       #### check and run before_change hook
-      Rex::Hook::run_hook( file => "before_change", @_ );
+      Rex::Hook::run_hook( file => "before_change", $file, %{$option} );
       ##############################
 
       my $fh = file_write($file);
@@ -654,7 +670,7 @@ sub file {
       );
 
       #### check and run after_change hook
-      Rex::Hook::run_hook( file => "after_change", @_, $__ret );
+      Rex::Hook::run_hook( file => "after_change", $file, %{$option}, $__ret );
       ##############################
 
     }
@@ -732,9 +748,14 @@ sub file {
   if ( $__ret->{changed} == 1 && $on_change_done == 0 ) {
     &$on_change($file);
   }
+  elsif ( $__ret->{changed} == 0 ) {
+    Rex::Logger::debug(
+      "File $file has not been changed... Running on_no_change");
+    &$on_no_change($file);
+  }
 
   #### check and run after hook
-  Rex::Hook::run_hook( file => "after", @_, $__ret );
+  Rex::Hook::run_hook( file => "after", $file, %{$option}, $__ret );
   ##############################
 
   Rex::get_current_connection()->{reporter}
@@ -743,6 +764,20 @@ sub file {
   return $__ret->{changed};
 }
 
+sub get_tmp_file_name {
+  my $file = shift;
+
+  my $dirname  = dirname($file);
+  my $filename = ".rex.tmp." . basename($file);
+
+  my $tmp_file_name =
+      $dirname eq '.'
+    ? $filename
+    : Rex::Helper::File::Spec->catfile( $dirname, $filename );
+
+  return $tmp_file_name;
+}
+
 =head2 file_write($file_name)
 
 This function opens a file for writing (it will truncate the file if it already exists). It returns a Rex::FS::File object on success.
@@ -756,7 +791,7 @@ On failure it will die.
  
  # catch an error
  if($@) {
-   print "An error occured. $@.\n";
+   print "An error occurred. $@.\n";
  }
  
  # work with the filehandle
@@ -813,7 +848,7 @@ On failure it will die.
  
  # catch an error
  if($@) {
-   print "An error occured. $@.\n";
+   print "An error occurred. $@.\n";
  }
  
  # work with the filehandle
@@ -942,7 +977,7 @@ OUT:
 
 =head2 delete_lines_according_to($search, $file, @options)
 
-This is the successor of the delete_lines_matching() function. This function also allows the usage of an on_change hook.
+This is the successor of the delete_lines_matching() function. This function also allows the usage of on_change and on_no_change hooks.
 
 It will search for $search in $file and remove the found lines. If on_change hook is present it will execute this if the file was changed.
 
@@ -959,8 +994,9 @@ sub delete_lines_according_to {
   my ( $search, $file, @options ) = @_;
   $file = resolv_path($file);
 
-  my $option    = {@options};
-  my $on_change = $option->{on_change} || undef;
+  my $option       = {@options};
+  my $on_change    = $option->{on_change}    || undef;
+  my $on_no_change = $option->{on_no_change} || undef;
 
   my ( $old_md5, $new_md5 );
 
@@ -970,11 +1006,14 @@ sub delete_lines_according_to {
 
   delete_lines_matching( $file, $search );
 
-  if ($on_change) {
+  if ( $on_change || $on_no_change ) {
     $new_md5 = md5($file);
 
     if ( $old_md5 ne $new_md5 ) {
-      &$on_change($file);
+      &$on_change($file) if $on_change;
+    }
+    else {
+      &$on_no_change($file) if $on_no_change;
     }
   }
 
@@ -1021,8 +1060,11 @@ found, it will be updated. Otherwise, it will be appended.
      line  => "mygroup:*:100:myuser3,myuser4",
      regexp => qr{^mygroup},
      on_change => sub {
-                say "file was changed, do something.";
-              };
+       say "file was changed, do something.";
+     },
+     on_no_change => sub {
+       say "file was not changed, do something.";
+     };
  };
 
 =cut
@@ -1039,7 +1081,7 @@ sub _append_or_update {
   my ( $new_line, @m );
 
   # check if parameters are in key => value format
-  my ( $option, $on_change );
+  my ( $option, $on_change, $on_no_change );
 
   Rex::get_current_connection()->{reporter}
     ->report_resource_start( type => $action, name => $file );
@@ -1060,17 +1102,22 @@ sub _append_or_update {
     elsif ( ref $option->{regexp} eq "ARRAY" ) {
       @m = @{ $option->{regexp} };
     }
-    $on_change = $option->{on_change} || undef;
+    $on_change    = $option->{on_change}    || undef;
+    $on_no_change = $option->{on_no_change} || undef;
     1;
   } or do {
     ( $new_line, @m ) = @_;
 
-    # check if something in @m (the regexpes) is named on_change
-    for ( my $i = 0 ; $i < $#m ; $i++ ) {
-      if ( $m[$i] eq "on_change" && ref( $m[ $i + 1 ] ) eq "CODE" ) {
-        $on_change = $m[ $i + 1 ];
-        splice( @m, $i, 2 );
-        last;
+    # check if something in @m (the regexpes) is named on_change or on_no_change
+    for my $option ( [ on_change => \$on_change ],
+      [ on_no_change => \$on_no_change ] )
+    {
+      for ( my $i = 0 ; $i < $#m ; $i++ ) {
+        if ( $m[$i] eq $option->[0] && ref( $m[ $i + 1 ] ) eq "CODE" ) {
+          ${ $option->[1] } = $m[ $i + 1 ];
+          splice( @m, $i, 2 );
+          last;
+        }
       }
     }
   };
@@ -1108,32 +1155,47 @@ sub _append_or_update {
         $match = qr{$match};
       }
       if ( $content->[$line] =~ $match ) {
-        return 0 if $action eq 'append_if_no_such_line';
-        $content->[$line] = "$new_line";
         $found = 1;
+        last if $action eq 'append_if_no_such_line';
+        $content->[$line] = "$new_line";
       }
     }
   }
 
-  push @$content, "$new_line" unless $found;
+  my $new_md5;
+  if ( $action eq 'append_if_no_such_line' && $found ) {
+    $new_md5 = $old_md5;
+  }
+  else {
+    push @$content, "$new_line" unless $found;
 
-  file $file,
-    content => join( "\n", @$content ),
-    owner   => $stat{uid},
-    group   => $stat{gid},
-    mode    => $stat{mode};
+    file $file,
+      content => join( "\n", @$content ),
+      owner   => $stat{uid},
+      group   => $stat{gid},
+      mode    => $stat{mode};
+    $new_md5 = md5($file);
+  }
 
-  my $new_md5 = md5($file);
-
-  if ($on_change) {
+  if ( $on_change || $on_no_change ) {
     if ( $old_md5 && $new_md5 && $old_md5 ne $new_md5 ) {
-      $old_md5 ||= "";
+      if ($on_change) {
+        $old_md5 ||= "";
+        $new_md5 ||= "";
+
+        Rex::Logger::debug("File $file has been changed... Running on_change");
+        Rex::Logger::debug("old: $old_md5");
+        Rex::Logger::debug("new: $new_md5");
+        &$on_change($file);
+      }
+    }
+    elsif ($on_no_change) {
       $new_md5 ||= "";
 
-      Rex::Logger::debug("File $file has been changed... Running on_change");
-      Rex::Logger::debug("old: $old_md5");
-      Rex::Logger::debug("new: $new_md5");
-      &$on_change($file);
+      Rex::Logger::debug(
+        "File $file has not been changed (md5 $new_md5)... Running on_no_change"
+      );
+      &$on_no_change($file);
     }
   }
 
@@ -1256,6 +1318,8 @@ Search some string in a file and replace it.
     multiline => TRUE;
  };
 
+Like similar file management commands, it also supports C<on_change> and C<on_no_change> hooks.
+
 =cut
 
 sub sed {
@@ -1273,7 +1337,8 @@ sub sed {
     $options = {@option};
   }
 
-  my $on_change = $options->{"on_change"} || undef;
+  my $on_change    = $options->{"on_change"}    || undef;
+  my $on_no_change = $options->{"on_no_change"} || undef;
 
   my @content;
 
@@ -1293,11 +1358,12 @@ sub sed {
 
   my $ret = file(
     $file,
-    content   => join( "\n", @content ),
-    on_change => $on_change,
-    owner     => $stat{uid},
-    group     => $stat{gid},
-    mode      => $stat{mode}
+    content      => join( "\n", @content ),
+    on_change    => $on_change,
+    on_no_change => $on_no_change,
+    owner        => $stat{uid},
+    group        => $stat{gid},
+    mode         => $stat{mode}
   );
 
   Rex::get_current_connection()->{reporter}
