@@ -36,10 +36,11 @@ With this module you can manage user and groups.
 
 package Rex::Commands::User;
 
+use 5.010001;
 use strict;
 use warnings;
 
-our $VERSION = '1.9.0'; # VERSION
+our $VERSION = '1.13.2'; # VERSION
 
 require Rex::Exporter;
 use Rex::Commands::Fs;
@@ -126,6 +127,20 @@ sub account {
 
 Create or update a user.
 
+This function supports the following L<hooks|Rex::Hook>:
+
+=over 4
+
+=item before
+
+This gets executed before the user is created. All original parameters are passed to it.
+
+=item after
+
+This gets executed after the user is created. All original parameters, and the user's C<UID> are passed to it.
+
+=back
+
 =cut
 
 sub create_user {
@@ -311,11 +326,17 @@ sub group_resource {
     Rex::get_current_connection()->{reporter}
       ->report_resource_start( type => "group", name => $group_name );
 
+    my $gid = get_gid($group_name);
+
     if ( $option{ensure} eq "present" ) {
-      Rex::Commands::User::create_group( $group_name, %option );
+      if ( !defined $gid ) {
+        Rex::Commands::User::create_group( $group_name, %option );
+      }
     }
     elsif ( $option{ensure} eq "absent" ) {
-      Rex::Commands::User::delete_group($group_name);
+      if ( defined $gid ) {
+        Rex::Commands::User::delete_group($group_name);
+      }
     }
     else {
       die "Unknown 'ensure' value. Valid values are 'present' and 'absent'.";
