@@ -6,10 +6,11 @@
 
 package Rex::Interface::Fs::SSH;
 
+use 5.010001;
 use strict;
 use warnings;
 
-our $VERSION = '1.9.0'; # VERSION
+our $VERSION = '1.13.2'; # VERSION
 
 use Rex::Helper::File::Stat;
 use Rex::Helper::Encode;
@@ -68,7 +69,7 @@ sub is_dir {
 
   defined $stat && defined $stat->{mode}
     ? return Rex::Helper::File::Stat->S_ISDIR( $stat->{mode} )
-    : return undef; ## no critic ProhibitExplicitReturnUndef
+    : return undef;
 }
 
 sub is_file {
@@ -87,7 +88,7 @@ sub is_file {
       || Rex::Helper::File::Stat->S_ISCHR( $stat->{mode} )
       || Rex::Helper::File::Stat->S_ISFIFO( $stat->{mode} )
       || Rex::Helper::File::Stat->S_ISSOCK( $stat->{mode} ) )
-    : return undef; ## no critic ProhibitExplicitReturnUndef
+    : return undef;
 }
 
 sub unlink {
@@ -135,7 +136,7 @@ sub stat {
   my $sftp = Rex::get_sftp();
   my %ret  = $sftp->stat($file);
 
-  if ( !%ret ) { return undef; } ## no critic ProhibitExplicitReturnUndef
+  if ( !%ret ) { return undef; }
 
   $ret{'mode'} =
     sprintf( "%04o", Rex::Helper::File::Stat->S_IMODE( $ret{'mode'} ) );
