@@ -6,10 +6,11 @@
 
 package Rex::Resource::Common;
 
+use 5.010001;
 use strict;
 use warnings;
 
-our $VERSION = '1.12.0'; # VERSION
+our $VERSION = '1.13.2'; # VERSION
 
 require Exporter;
 require Rex::Config;
@@ -96,19 +97,16 @@ sub resource {
   if (!$class->can($name)
     && $name_save =~ m/^[a-zA-Z_][a-zA-Z0-9_]+$/ )
   {
-    Rex::Logger::debug("Registering resource: ${class}::$name_save");
+    if ( $class ne "main" && $class ne "Rex::CLI" ) {
+
+      # if not in main namespace, register the task as a sub
+      Rex::Logger::debug(
+        "Registering resource (not main namespace): ${class}::$name_save");
+    }
+    else {
+      Rex::Logger::debug("Registering resource: ${class}::$name_save");
+    }
 
-    my $code            = $_[-2];
-    my $ref_to_resource = qualify_to_ref( $name_save, $class );
-    *{$ref_to_resource} = $func;
-  }
-  elsif ( ( $class ne "main" && $class ne "Rex::CLI" )
-    && !$class->can($name_save)
-    && $name_save =~ m/^[a-zA-Z_][a-zA-Z0-9_]+$/ )
-  {
-    # if not in main namespace, register the task as a sub
-    Rex::Logger::debug(
-      "Registering resource (not main namespace): ${class}::$name_save");
     my $code            = $_[-2];
     my $ref_to_resource = qualify_to_ref( $name_save, $class );
     *{$ref_to_resource} = $func;
